<?php

defined( 'ABSPATH' ) || exit;

/**
 * Handle Variation Switcher Ajax Event.
 * 
 * @class VRN_Ajax
 * @package Class
 */
class VRN_Ajax {

	/**
	 * Init VRN_Ajax.
	 */
	public static function init() {
		//Get Ajax Events.
		$prefix      = VRN_PREFIX;
		$ajax_events = array(
			'switch_variation'              => true,
			'json_search_variable_products' => false,
		);

		foreach ( $ajax_events as $ajax_event => $nopriv ) {
			add_action( "wp_ajax_{$prefix}{$ajax_event}", __CLASS__ . "::{$ajax_event}" );

			if ( $nopriv ) {
				add_action( "wp_ajax_nopriv_{$prefix}{$ajax_event}", __CLASS__ . "::{$ajax_event}" );
			}
		}
	}

	/**
	 *  Update the variations in cart.
	 */
	public static function switch_variation() {
		check_ajax_referer( 'vrn-switch-variation', 'security' );

		try {
			$posted = $_POST;
			if ( ! isset( $posted[ 'data' ] ) ) {
				throw new Exception( __( 'Sorry!!! Can not switch Variation', 'variation-switcher-in-cart-for-woocommerce' ) );
			}

			$data = wp_parse_args( wp_unslash( $posted[ 'data' ] ) );

			if ( 0 === absint( $data[ 'product_id' ] ) || 0 === absint( $data[ 'variation_id' ] ) ) {
				throw new Exception( __( 'Something went wrong!!!', 'variation-switcher-in-cart-for-woocommerce' ) );
			}

			$variations         = array();
			$previous_cart_key  = $data[ 'vrn_cart_item_key' ];
			$previous_cart_item = WC()->cart->get_cart_item( $previous_cart_key );

			/**
			 * Get add to cart variation item data.
			 * 
			 * @since 1.0
			 */
			$cart_item_data = ( array ) apply_filters( 'vrn_add_cart_item_data', array(), $previous_cart_key, $previous_cart_item );

			foreach ( $data as $key => $value ) {
				if ( 'attribute_' !== substr( $key, 0, 10 ) ) {
					continue;
				}

				$variations[ sanitize_title( wp_unslash( $key ) ) ] = wp_unslash( $value );
			}

			//Add the new variation product to the cart.
			$was_added_to_cart = WC()->cart->add_to_cart( $data[ 'product_id' ], $previous_cart_item[ 'quantity' ], $data[ 'variation_id' ], $variations, $cart_item_data );

			if ( is_string( $was_added_to_cart ) ) {
				if ( $previous_cart_key === $was_added_to_cart ) {
					// Set last known Qty.
					WC()->cart->set_quantity( $previous_cart_key, $previous_cart_item[ 'quantity' ], true );
				} else {
					//Remove old variation product from the cart.
					WC()->cart->remove_cart_item( $previous_cart_key );

					//For variation update notice success message.
					wc_add_notice( wp_kses_post( trim( get_option( VRN_PREFIX . 'success_message' ) ) ), 'success' );
				}

				wp_send_json_success();
			}
		} catch ( Exception $ex ) {
			wp_send_json_error( array( 'error' => esc_html( $ex->getMessage() ) ) );
		}

		wp_die();
	}

	/**
	 * Search Variable Products.
	 */
	public static function json_search_variable_products() {
		check_ajax_referer( 'search-products', 'security' );

		if ( empty( $term ) && isset( $_GET[ 'term' ] ) ) {
			$term = ( string ) wc_clean( wp_unslash( $_GET[ 'term' ] ) );
		}

		if ( empty( $term ) ) {
			wp_die();
		}

		if ( ! empty( $_GET[ 'limit' ] ) ) {
			$limit = absint( $_GET[ 'limit' ] );
		} else {
			$limit = 30;
		}

		$include_ids = ! empty( $_GET[ 'include' ] ) ? array_map( 'absint', ( array ) wp_unslash( $_GET[ 'include' ] ) ) : array();
		$exclude_ids = ! empty( $_GET[ 'exclude' ] ) ? array_map( 'absint', ( array ) wp_unslash( $_GET[ 'exclude' ] ) ) : array();

		$exclude_types = array();
		if ( ! empty( $_GET[ 'exclude_type' ] ) ) {
			// Support both comma-delimited and array format inputs.
			$exclude_types = wp_unslash( wc_clean( $_GET[ 'exclude_type' ] ) ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			if ( ! is_array( $exclude_types ) ) {
				$exclude_types = explode( ',', $exclude_types );
			}

			// Sanitize the excluded types against valid product types.
			foreach ( $exclude_types as &$exclude_type ) {
				$exclude_type = strtolower( trim( $exclude_type ) );
			}
			$exclude_types = array_intersect(
					array_merge( array( 'variation' ), array_keys( wc_get_product_types() ) ), $exclude_types
			);
		}

		$data_store = WC_Data_Store::load( 'product' );
		$ids        = $data_store->search_products( $term, '', false, false, $limit, $include_ids, $exclude_ids );

		$products = array();

		foreach ( $ids as $id ) {
			$product_object = wc_get_product( $id );

			if ( ! wc_products_array_filter_readable( $product_object ) ) {
				continue;
			}

			if ( ! in_array( $product_object->get_type(), _vrn()->supported_product_types() ) ) {
				continue;
			}

			$formatted_name = $product_object->get_formatted_name();
			$managing_stock = $product_object->managing_stock();

			if ( in_array( $product_object->get_type(), $exclude_types, true ) ) {
				continue;
			}

			if ( $managing_stock && ! empty( $_GET[ 'display_stock' ] ) ) {
				$stock_amount   = $product_object->get_stock_quantity();
				/* Translators: %d stock amount */
				$formatted_name .= ' &ndash; ' . sprintf( __( 'Stock: %d', 'variation-switcher-in-cart-for-woocommerce' ), wc_format_stock_quantity_for_display( $stock_amount, $product_object ) );
			}

			$products[ $product_object->get_id() ] = rawurldecode( wp_strip_all_tags( $formatted_name ) );
		}

		wp_send_json( $products );
	}
}

VRN_Ajax::init();
