<?php

defined( 'ABSPATH' ) || exit;

/**
 * Variation Switcher Display Handler
 * 
 * @class VRN_Display
 * @package Class
 */
class VRN_Display {
	/*
	 * Skip the variation to switch.
	 */

	public static $skip_variation_id = null;

	/**
	 * Init VRN_Display.
	 */
	public static function init() {
		add_action( 'woocommerce_after_cart_item_name', __CLASS__ . '::render_in_cart', 1, 2 );
		add_filter( 'woocommerce_checkout_cart_item_quantity', __CLASS__ . '::render_in_checkout', 1, 3 );
		add_filter( 'woocommerce_cart_contents_changed', __CLASS__ . '::alter_cart_contents_order', 10, 1 );
		add_filter( 'vrn_add_cart_item_data', __CLASS__ . '::add_previous_cart_item_position', 99, 2 );

		if ( 'no' === get_option( VRN_PREFIX . 'display_selected_variation', 'no' ) ) {
			add_filter( 'woocommerce_variation_is_visible', __CLASS__ . '::skip_variation', 999, 2 );
		}
	}

	/**
	 * Get the Variation Switcher Template.
	 * 
	 * @param array $cart_item
	 * @param string $item_key
	 * @param bool $echo
	 */
	public static function get_template( $cart_item, $item_key, $echo = true ) {
		$product = wc_get_product( $cart_item[ 'product_id' ] );
		if ( ! $product ) {
			return;
		}

		//Check whether the Product type is eligible or Not.
		if ( ! in_array( $product->get_type(), _vrn()->supported_product_types() ) ) {
			return;
		}

		//Check the Product is eligible to Switch Variation.
		if ( ! _vrn_is_switchable( $cart_item[ 'product_id' ] ) ) {
			return;
		}

		/**
		 * Filter to display variation switcher for third party plugins in cart and checkout.
		 * 
		 * @since 1.0
		 */
		if ( ! apply_filters( 'vrn_display_variation_switcher', true ) ) {
			return;
		}

		//To skip the variation already in cart.
		self::$skip_variation_id = $cart_item[ 'variation_id' ];

		$available_variations = $product->get_available_variations();

		self::$skip_variation_id = null;

		// Enqueue variation scripts.
		wp_enqueue_script( 'wc-add-to-cart-variation' );

		// Load the template.
		ob_start();
		wc_get_template( 'switcher.php', array(
			'product'              => $product,
			'available_variations' => $available_variations,
			'cart_item_key'        => $item_key,
				), VRN_BASENAME_DIR, VRN_TEMPLATE_PATH );

		if ( $echo ) {
			return ob_end_flush();
		} else {
			return ob_get_clean();
		}
	}

	/**
	 * Render the variation switcher in Cart.
	 * 
	 * @param array $cart_item
	 * @param string $item_key
	 * 
	 */
	public static function render_in_cart( $cart_item, $item_key ) {
		// Check the user is eligible.
		if ( ! current_user_can( 'vrn-switch-variation' ) ) {
			return;
		}

		self::get_template( $cart_item, $item_key );
	}

	/**
	 * Render the Variation Switcher in Checkout Page.
	 * 
	 * @param int $return
	 * @param array $cart_item
	 * @param string $cart_item_key
	 * @return string
	 */
	public static function render_in_checkout( $return, $cart_item, $cart_item_key ) {
		// Check the user is eligible.
		if ( ! current_user_can( 'vrn-switch-variation' ) ) {
			return $return;
		}

		if ( _vrn()->is_enabled_in_checkout() ) {
			$template = self::get_template( $cart_item, $cart_item_key, false );

			if ( ! is_null( $template ) ) {
				$return .= $template;
			}
		}

		return $return;
	}

	/**
	 * Skip the variation which was already in cart.
	 * 
	 * @param boolean $bool
	 * @param int $variation_id
	 * @param int $variable_id
	 * @param array $variation
	 * @return boolean
	 */
	public static function skip_variation( $bool, $variation_id ) {
		if ( is_null( self::$skip_variation_id ) ) {
			return $bool;
		}

		if ( absint( self::$skip_variation_id ) === absint( $variation_id ) ) {
			return false;
		}

		return $bool;
	}

	/**
	 * Alter the cart contents order.
	 * 
	 * @return array
	 * */
	public static function alter_cart_contents_order( $cart_contents ) {
		// Return the same cart content if contents is empty.
		if ( empty( $cart_contents ) ) {
			return $cart_contents;
		}

		$newly_switched_cart_contents = array();
		foreach ( $cart_contents as $key => $values ) {
			if ( isset( $values[ 'vrn_prev_cart_item_position' ] ) ) {
				$postion = $values[ 'vrn_prev_cart_item_position' ];

				unset( $values[ 'vrn_prev_cart_item_position' ] );
				$newly_switched_cart_contents[ $key ] = $values;
			}
		}

		if ( empty( $newly_switched_cart_contents ) ) {
			return $cart_contents;
		}

		// Replace newly switched cart contents to the corresponding position without preserving cart keys.
		array_splice( $cart_contents, $postion, 0, $newly_switched_cart_contents );

		// To preserve cart keys, we are doing here.
		$altered_cart_contents = array();
		foreach ( $cart_contents as $key => $values ) {
			if ( is_numeric( $key ) ) {
				$key = $values[ 'key' ];
			}

			$altered_cart_contents[ $key ] = $values;
		}

		return $altered_cart_contents;
	}

	/**
	 * Add previous cart item position to newly switched cart item data.
	 * 
	 * @param array $cart_item_data
	 * @param string $previous_cart_key
	 * @return array
	 */
	public static function add_previous_cart_item_position( $cart_item_data, $previous_cart_key ) {
		$cart_item_data[ 'vrn_prev_cart_item_position' ] = array_search( $previous_cart_key, array_keys( WC()->cart->get_cart_contents() ) );
		return $cart_item_data;
	}
}

VRN_Display::init();
