<?php

defined( 'ABSPATH' ) || exit;

/**
 * Initiate Plugin Core class.
 * 
 * @class WC_Variation_Switcher_In_Cart
 * @package Class
 */
final class WC_Variation_Switcher_In_Cart {

	/**
	 * Plugin version.
	 * 
	 * @var string 
	 */
	public $version = '1.9.0';

	/**
	 * Plugin prefix.
	 * 
	 * @var string 
	 */
	public $prefix = '_vrn_';

	/**
	 * Is Variation Switcher in cart enabled for guests ?
	 * 
	 * @var bool
	 */
	protected $is_enabled_for_guests;

	/**
	 * Is Variation Switcher in cart enabled for users ?
	 * 
	 * @var bool 
	 */
	protected $is_enabled_for_users;

	/*
	 * Variation Switcher available in Checkout?
	 */
	protected $is_enabled_in_checkout;

	/**
	 * The single instance of the class.
	 */
	protected static $instance = null;

	/**
	 * WC_Variation_Switcher_In_Cart constructor.
	 */
	public function __construct() {
		add_action( 'plugins_loaded', array( $this, 'plugins_loaded' ) );
		add_action( 'admin_notices', array( $this, 'plugin_dependencies_notice' ) );
	}

	/**
	 * Cloning is forbidden.
	 */
	public function __clone() {
		_doing_it_wrong( __FUNCTION__, esc_html__( 'Cloning is forbidden.', 'variation-switcher-in-cart-for-woocommerce' ), '1.0' );
	}

	/**
	 * Unserializing instances of this class is forbidden.
	 */
	public function __wakeup() {
		_doing_it_wrong( __FUNCTION__, esc_html__( 'Unserializing instances of this class is forbidden.', 'variation-switcher-in-cart-for-woocommerce' ), '1.0' );
	}

	/**
	 * Main WC_Variation_Switcher_In_Cart Instance.
	 * Ensures only one instance of WC_Variation_Switcher_In_Cart is loaded or can be loaded.
	 * 
	 * @return WC_Variation_Switcher_In_Cart - Main instance.
	 */
	public static function instance() {
		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	/**
	 * Check whether the plugin dependencies met.
	 * 
	 * @return bool|string True on Success
	 */
	private function plugin_dependencies_met( $return_dep_notice = false ) {
		$return = false;

		// WC check.
		if ( ! function_exists( 'WC' ) ) {
			if ( $return_dep_notice ) {
				$install_url = wp_nonce_url( add_query_arg( array( 'action' => 'install-plugin', 'plugin' => 'woocommerce' ), admin_url( 'update.php' ) ), 'install-plugin_woocommerce' );
				// translators: 1$-2$: opening and closing <strong> tags, 3$-4$: link tags, takes to woocommerce plugin on wp.org, 5$-6$: opening and closing link tags, leads to plugins.php in admin
				$return      = sprintf( esc_html__( '%1$sVariation Switcher is inactive.%2$s The %3$sWooCommerce plugin%4$s must be active for Variation Switcher to work. Please %5$sinstall & activate WooCommerce &raquo;%6$s', 'variation-switcher-in-cart-for-woocommerce' ), '<strong>', '</strong>', '<a href="http://wordpress.org/extend/plugins/woocommerce/">', '</a>', '<a href="' . esc_url( $install_url ) . '">', '</a>' );
			}

			return $return;
		}

		return true;
	}

	/**
	 * When WP has loaded all plugins, trigger the `vrn_loaded` hook.
	 */
	public function plugins_loaded() {
		if ( true !== $this->plugin_dependencies_met() ) {
			return; // Return to stop the existing function to be call 
		}

		$this->define_constants();
		$this->include_files();
		$this->init_hooks();
		$this->load_plugin_textdomain();

		/**
		 * Trigger after plugin is loaded.
		 * 
		 * @since 1.0
		 */
		do_action( 'vrn_loaded' );
	}

	/**
	 * Output a admin notice when plugin dependencies not met.
	 */
	public function plugin_dependencies_notice() {
		$return = $this->plugin_dependencies_met( true );

		if ( true !== $return && current_user_can( 'activate_plugins' ) ) {
			$dependency_notice = $return;
			printf( '<div class="error"><p>%s</p></div>', wp_kses_post( $dependency_notice ) );
		}
	}

	/**
	 * Define constants.
	 */
	private function define_constants() {
		$this->define( 'VRN_DIR', plugin_dir_path( VRN_FILE ) );
		$this->define( 'VRN_BASENAME', plugin_basename( VRN_FILE ) );
		$this->define( 'VRN_BASENAME_DIR', trailingslashit( dirname( VRN_BASENAME ) ) );
		$this->define( 'VRN_TEMPLATE_PATH', VRN_DIR . 'templates/' );
		$this->define( 'VRN_URL', untrailingslashit( plugins_url( '/', VRN_FILE ) ) );
		$this->define( 'VRN_VERSION', $this->version );
		$this->define( 'VRN_PREFIX', $this->prefix );
	}

	/**
	 * Define constant if not already set.
	 *
	 * @param string      $name  Constant name.
	 * @param string|bool $value Constant value.
	 */
	private function define( $name, $value ) {
		if ( ! defined( $name ) ) {
			define( $name, $value );
		}
	}

	/**
	 * Is frontend request ?
	 *
	 * @return bool
	 */
	private function is_frontend() {
		return ( ! is_admin() || defined( 'DOING_AJAX' ) ) && ! defined( 'DOING_CRON' );
	}

	/**
	 * Check if our plugin enabled for logged-in users.
	 * 
	 * @return bool
	 */
	public function is_enabled_for_users() {
		if ( ! is_bool( $this->is_enabled_for_users ) ) {
			$this->is_enabled_for_users = 'yes' === get_option( VRN_PREFIX . 'enabled_for_loged_in_users' ) ? true : false;
		}

		return $this->is_enabled_for_users;
	}

	/**
	 * Check if our plugin enabled for guests.
	 *
	 * @return bool
	 */
	public function is_enabled_for_guests() {
		if ( ! is_bool( $this->is_enabled_for_guests ) ) {
			$this->is_enabled_for_guests = 'yes' === get_option( VRN_PREFIX . 'enabled_for_guest_users' ) ? true : false;
		}

		return $this->is_enabled_for_guests;
	}

	/**
	 * Check if Variation Switcher is enabled in Checkout Page.
	 * 
	 * @return bool
	 */
	public function is_enabled_in_checkout() {
		if ( ! is_bool( $this->is_enabled_in_checkout ) ) {
			$this->is_enabled_in_checkout = 'yes' === get_option( VRN_PREFIX . 'enabled_in_checkout' ) ? true : false;
		}

		return $this->is_enabled_in_checkout;
	}

	/**
	 * Returns the supported product types.
	 * 
	 * @return array
	 */
	public function supported_product_types() {
		/**
		 * Get supported product types.
		 * 
		 * @since 1.0
		 */
		return ( array ) apply_filters( 'vrn_supported_product_types', array( 'variable', 'variable-subscription' ) );
	}

	/**
	 * Include required core files used in admin and on the frontend.
	 */
	private function include_files() {
		//Core functions.
		include_once 'vrn-core-functions.php';

		//Core classes.
		include_once 'class-vrn-install.php';
		include_once 'class-vrn-ajax.php';
		include_once 'class-vrn-enqueues.php';
		include_once 'privacy/class-vrn-privacy.php';

		if ( is_admin() ) {
			include_once 'admin/class-vrn-admin.php';
		}

		if ( $this->is_frontend() && ( $this->is_enabled_for_users() || $this->is_enabled_for_guests() ) ) {
			include_once 'class-vrn-display.php';
		}
	}

	/**
	 * Hook into actions and filters.
	 */
	private function init_hooks() {
		register_activation_hook( VRN_FILE, array( 'VRN_Install', 'install' ) );
		add_filter( 'user_has_cap', array( $this, 'customer_has_capability' ), 10, 3 );
	}

	/**
	 * Check the user capability.
	 * 
	 * @param type $allcaps
	 * @param type $caps
	 * @param type $args
	 * @return boolean
	 */
	public function customer_has_capability( $allcaps, $caps, $args ) {
		if ( isset( $caps[ 0 ] ) ) {
			switch ( $caps[ 0 ] ) {
				case 'vrn-switch-variation':
					$user_id = absint( $args[ 1 ] );

					if ( 0 === $user_id ) {
						if ( $this->is_enabled_for_guests() ) {
							$allcaps[ 'vrn-switch-variation' ] = true;
						}
					} elseif ( $this->is_enabled_for_users() ) {
							$allcaps[ 'vrn-switch-variation' ] = true;
					}
					break;
			}
		}

		return $allcaps;
	}

	/**
	 * Load Localization files.
	 */
	public function load_plugin_textdomain() {
		if ( function_exists( 'determine_locale' ) ) {
			$locale = determine_locale();
		} else {
			$locale = is_admin() ? get_user_locale() : get_locale();
		}

		/**
		 * Get plugin locale.
		 * 
		 * @since 1.0
		 */
		$locale = apply_filters( 'plugin_locale', $locale, 'variation-switcher-in-cart-for-woocommerce' );

		unload_textdomain( 'variation-switcher-in-cart-for-woocommerce' );
		load_textdomain( 'variation-switcher-in-cart-for-woocommerce', WP_LANG_DIR . '/variation-switcher-in-cart-for-woocommerce/variation-switcher-in-cart-for-woocommerce-' . $locale . '.mo' );
		load_plugin_textdomain( 'variation-switcher-in-cart-for-woocommerce', false, VRN_BASENAME_DIR . 'languages' );
	}
}
